%    This file is part of the psPack simulation package from the
%    doctoral dissertation "A DAE Formulation for Multi-Zone
%    Application to CVCP Engines" by Per Öberg. 
%    The dissertation is available at 
%    http://urn.kb.se/resolve?urn=urn:nbn:se:liu:diva-18120
%
%    Copyright (C) 2013  Per Öberg <per.oberg@liu.se>
%
%    This program is free software; you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation; either version 2 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License along
%    with this program; if not, write to the Free Software Foundation, Inc.,
%    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

function [dXpT dXgc] = psDX_Kinetic(p,T,V,Xgc,dQ,TUp,XgcUp,dmUp,V_bar_Up,TDown,XgcDown,dmDown,V_bar_Down);
% function dX = psDX(XpVT,Xgc,dV,dm,dQ,extFlow);
%    XpT    = [p, T]'   2x1
%    Xgc     = [x_uf x_ua x_bf  x_ba]         1xK 
%    dQ      = Heat transfer from resp zone   1x1
%  External mass flow: 
%    TDown    = Temperture outside the control volume  1x1
%    XgcDown  = Gas composition upstreams              1xK
%    dmDown   = Mass flow TO the control volume        1x1
%      * Note the sign/direction convention
%    TUp    = Temperture outside the control volume  1x1
%    XgcUp  = Gas composition upstreams              1xK
%    dmUp   = Mass flow TO the control volume        1x1
%      * Note the sign/direction convention
% Output:
%    dXpVT = 2x1
%    dXgc  = 1xK
%
% Author: Per Öberg (renewed implementation of old psPack with multiple
%                    authors)
%
% Changelog: 
%  2008-04-16 - Created

%V_bar_Up = 0;
%V_bar_Down= 0;

global psSimStruct
K = psSimStruct.psThermProp.K;

% No data in X can be <0, physical limits for volume,
% pressure and temperature. This could generate a warning but since
% this will most probably only happen for calculations that will be
% discarded it does not seem necessary.

 

% This is where we get the thermodynamic properties
% We need:
% 1) For each zone everything is needed 
outputs = ones(1,9);
[h cp cv R dRdp dRdT dudp dudX dRdX]=psThermProp(p,T,Xgc,outputs);

% 2) for external zones only those who are from in-flows enthalpy is
% realy needed. For now, however, the implementation calculates 
% everything no matter if it's already been calulated before.
T_hat_Up   = (dmUp <= 0 )*T(1)+(dmUp > 0 ).*TUp;
T_hat_Down = (dmDown <= 0 )*T(end)+(dmDown > 0 ).*TDown;

Xgc_hat_Up = (dmUp > 0)*XgcUp+(dmUp <= 0)*Xgc;
Xgc_hat_Down = (dmDown > 0)*XgcDown+(dmDown <= 0)*Xgc;


outputs = zeros(1,9);
outputs(1) = 1;
h_hat_Up = psThermProp(p,T_hat_Up,Xgc_hat_Up,outputs);
h_hat_Down = psThermProp(p,T_hat_Down,Xgc_hat_Down,outputs);
    
% Calculate dXgc as soon as possible
mdXgc=(Xgc_hat_Up-Xgc)*dmUp*(dmUp>0);
mdXgc=mdXgc+(Xgc_hat_Down-Xgc)*dmDown*(dmDown>0);
%dXgc=mdXgc/m;
dXgc=R*T/(p*V)*mdXgc;

%dXgc = dXgc';
if ~isreal(sum(dXgc))
    disp('Error in psDX.m: Complex dXgc')
end

% Build the matrix components and then finally the A-matrix. Note that it's
% the same for both "well stirred mixer" and "Well stirred reactor" since
% dRdp=dRdT=dudp=0 in the well stirred mixer case. The matrix is shifted
% for easier implementation
a = V-p.*V./R.*dRdp;
b =-p.*V./T-p.*V./R.*dRdT;
c = p.*V./R./T.*dudp;
d = p.*V./R./T.*cv;

Adet = a.*d-b.*c;
Ainv = [d./Adet -b./Adet ;...
        -c./Adet a./Adet];
% h = u + pv = u + RT => u = h - RT
B1      = dot(dRdX,mdXgc,2)*T+R*T*(dmUp+dmDown);
B2 = -dQ-dot(dudX,mdXgc,2)+(h_hat_Up-h+R*T+0.5*V_bar_Up^2)*dmUp+(h_hat_Down-h+R*T+0.5*V_bar_Down^2)*dmDown;

B = [B1;B2];

dXpT= Ainv*B;

